# global imports (dont't worry, red is normal)
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase
from db.entities.enum import Enum

from typing import List


class EnumRepository(MongoRepositoryBase):
    '''This is a repository for MongoDb.'''

    def __init__(self):
        super().__init__(netconst.BUSINESS_LOGIC_DB_HOSTNAME,
                         netconst.BUSINESS_LOGIC_DB_PORT,
                         'business-logic-db')

        self._enum_collection = 'enums'

    def all(self) -> List[Enum]:
        result = super().get_entries(
            self._enum_collection, projection={'_id': False})

        return [Enum.from_serializable_dict(row) for row in list(result)]

    def all_for_use_case(self, use_case: str) -> List[Enum]:
        result = super().get_entries(self._enum_collection, projection={
            '_id': False}, selection={"use_case": use_case})

        return [Enum.from_serializable_dict(row) for row in list(result)]

    def all_for_use_case_and_table(self, use_case: str, table: str) -> List[Enum]:
        result = super().get_entries(self._enum_collection, projection={
            '_id': False}, selection={"use_case": use_case, "table": table})

        return [Enum.from_serializable_dict(row) for row in list(result)]

    def all_instances(self, name: str, use_case: str, table: str) -> Enum:
        result = list(super().get_entries(self._enum_collection, selection={
                      "name": name, "use_case": use_case, "table": table}))

        return [Enum.from_serializable_dict(row) for row in result]

    def delete_all(self):
        collection = self._database[self._enum_collection]
        collection.delete_many({})

    def add(self, enum: Enum):
        super().insert_entry(self._enum_collection, enum.to_serializable_dict())

    def update_use_case(self, enum: Enum, use_case: str):
        collection = self._database[self._enum_collection]
        collection.update_one({"name": enum.name, "use_case": use_case, "table": enum.table}, {
                              "$set": enum.to_serializable_dict()})

    def update(self, enum: Enum):
        collection = self._database[self._enum_collection]
        collection.update_one({"name": enum.name, "use_case": enum.use_case, "table": enum.table}, {
                              "$set": enum.to_serializable_dict()})

    def delete(self, enum: Enum):
        collection = self._database[self._enum_collection]
        collection.delete_many(
            {"name": enum.name, "use_case": enum.use_case, "table": enum.table})
