#global imports
from db.entities.enum import Enum
from db.entities.layer_adapter import LayerAdapter
from db.repository import Repository
from db.table_repository import TableRepository
from db.use_case_repository import UseCaseRepository
from db.enum_repository import EnumRepository

import json
from flask import Response, request

table_repository = TableRepository()
use_case_repository = UseCaseRepository()
enum_repository = EnumRepository()

def all():
    return [enum.to_serializable_dict() for enum in enum_repository.all()]

def get_all_for_use_case(use_case: str):
    '''
    get all enums assigned to the given use_case
    '''
    use_case_repository.put(use_case)
    return [enum.to_serializable_dict() for enum in enum_repository.all_for_use_case(use_case)]

def one(use_case: str, table: str, name: str):
    '''
    fetch a single enum from the DB

    @params:
        use_case    - Required : String-identifier for the Use-Case the Enum belongs to
        table       - Required : unique identifier of the Table the Enum belongs to
        name        - Required : unique identifier for the Enum
    '''
    enum = enum_repository.one(name, use_case, table)

    if enum == None:
        return Response(status=404, response=f"Enum with name '{name}' does not exist!")

    return Response(status=200, response=json.dumps(enum.to_serializable_dict()))

def put_new(use_case: str, table: str, name: str, value: str):
    '''
    Put a new Enum to the DB. If the value for this enum already exists, nothing happens.

    @params:
        use_case    - Required : String-identifier for the Use-Case the Enum belongs to
        table       - Required : unique identifier of the Table the Enum belongs to
        name        - Required : unique identifier for the Enum
    '''
    existing_enums = enum_repository.all_instances(name, use_case, table)
    enum_target = None

    found = False
    for enum in existing_enums:
        if enum.value == value:
            enum_target = enum
            break

    if enum_target == None:
        enum_target = Enum(use_case, table, name, value, len(existing_enums))
        enum_repository.add(enum_target)

    return Response(status=200, response=json.dumps(enum_target.to_serializable_dict()))